@@ -111,182 +111,173 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
    /* Simple leapers */
    if (is_simple_leaper(flags)) {
       int index = get_leaper_index(flags);
-      uint8_t leaper_flags = movegen.leaper_description[index];
-      int dx = leaper_flags & 0x0f;
-      int dy = (leaper_flags >> 4) & 0x0f;
 
-      if (dx < 4 && dy < 4) {
-         n += snprintf(buffer+n, size-n, "%s%c", block, leaper_move_string[dx][dy]);
-      } else if (leaper_flags != 0xff) {
-         n += snprintf(buffer+n, size-n, "%s(%d, %d)", block, dx, dy);
-      } else {
-         /* Some combination of simple leapers, this is quite a bit more complicated.
-          * We use the following algorithm:
-          *  - Find the square where the leaper has maximum mobility.
-          *  - Find the simple leaper that has the largest overlap (test the King first)
-          *  - If the overlap is complete, we are done. If not, we need to extract directions.
-          *  - Repeat until all compound leapers have been identified.
-          */
+      /* Some combination of simple leapers, this is a bit complicated.
+       * We use the following algorithm:
+       *  - Find the square where the leaper has maximum mobility.
+       *  - Find the simple leaper that has the largest overlap (test the King first)
+       *  - If the overlap is complete, we are done. If not, we need to extract directions.
+       *  - Repeat until all compound leapers have been identified.
+       */
 
-         /* Find the square where the leaper has best mobility.
-          * We need this because the movement tables could have been hacked to exclude certain regions of the board.
-          */
-         bitboard_t<kind> leaper;
-         int square = 0;
-         int max_moves = 0;
-         for (int nn=0; nn<files*ranks; nn++) {
-            bitboard_t<kind> bb = is_aleaper(flags) ? movegen.aleaper[WHITE][index][nn] : movegen.leaper[index][nn];
-            int moves = bb.popcount();
-            if (moves > max_moves || (moves == max_moves && nn < 4*files)) {  /* Don't get stuck on back rank */
-               max_moves = moves;
-               square = nn;
-               leaper = bb;
+      /* Find the square where the leaper has best mobility.
+       * We need this because the movement tables could have been hacked to exclude certain regions of the board.
+       */
+      bitboard_t<kind> leaper;
+      int square = 0;
+      int max_moves = 0;
+      for (int nn=0; nn<files*ranks; nn++) {
+         bitboard_t<kind> bb = is_aleaper(flags) ? movegen.aleaper[WHITE][index][nn] : movegen.leaper[index][nn];
+         int moves = bb.popcount();
+         if (moves > max_moves || (moves == max_moves && nn < 4*files)) {  /* Don't get stuck on back rank */
+            max_moves = moves;
+            square = nn;
+            leaper = bb;
+         }
+      }
+
+      /* Construct symmetry masks */
+      if (is_aleaper(flags)) {
+         bitboard_t<kind> fd, bd, fa, ba;
+         int rank = unpack_rank(square);
+         int file = unpack_file(square);
+         int diag = bitboard_t<kind>::diagonal_nr[square];
+         int anti = bitboard_t<kind>::anti_diagonal_nr[square];
+
+         sym4[F] = bitboard_t<kind>::board_northward[rank];
+         sym4[B] = bitboard_t<kind>::board_southward[rank];
+         sym4[R] = bitboard_t<kind>::board_eastward[file];
+         sym4[L] = bitboard_t<kind>::board_westward[file];
+         sym4[V] = sym4[F] | sym4[B];
+         sym4[S] = sym4[R] | sym4[L];
+
+         sym4[RF4] = sym4[R] & sym4[L];
+         sym4[LF4] = sym4[L] & sym4[L];
+         sym4[RB4] = sym4[R] & sym4[B];
+         sym4[LB4] = sym4[L] & sym4[B];
+
+         sym8[FH] = sym4[F];
+         sym8[BH] = sym4[B];
+         sym8[LH] = sym4[L];
+         sym8[RH] = sym4[R];
+
+         fd.clear();
+         for (int n = diag+1; n<16; n++)
+            fd |= bitboard_t<kind>::board_diagonal[n];
+         fa.clear();
+         for (int n = anti+1; n<16; n++)
+            fa |= bitboard_t<kind>::board_antidiagonal[n];
+         bd.clear();
+         for (int n = 0; n < diag; n++)
+            bd |= bitboard_t<kind>::board_diagonal[n];
+         ba.clear();
+         for (int n = 0; n < anti; n++)
+            ba |= bitboard_t<kind>::board_antidiagonal[n];
+
+         sym8[FF] = fd & fa;
+         sym8[BB] = bd & ba;
+         sym8[LL] = fd & ba;
+         sym8[RR] = bd & fa;
+
+         sym8[FL] = sym8[FH] & sym8[LL];
+         sym8[LF] = sym8[LH] & sym8[FF];
+         sym8[FR] = sym8[FH] & sym8[RR];
+         sym8[RF] = sym8[RH] & sym8[FF];
+         sym8[BL] = sym8[BH] & sym8[LL];
+         sym8[LB] = sym8[LH] & sym8[BB];
+         sym8[BR] = sym8[BH] & sym8[RR];
+         sym8[RB] = sym8[RH] & sym8[BB];
+      }
+
+      /* Find which combination of simple leapers matches best.
+       * We try the non-atom king first.
+       */
+      bitboard_t<kind> kb = movegen.make_leaper_bitboard(square, 1, 1)|movegen.make_leaper_bitboard(square, 1, 0);
+      if (leaper == kb) {
+         n += snprintf(buffer+n, size-n, "K");
+         leaper.clear();
+      }
+
+      /* Loop over all fundamental leapers.
+       * FIXME: the loop can become infinite if the leaper is long (> 4) in any direction.
+       */
+      while (!leaper.is_empty()) {
+         bitboard_t<kind> best_leaper;
+         int best = -1;
+         int best_count = 0;
+         for (int nn = 0; leaper_description[nn].txt; nn++) {
+            int dx = leaper_description[nn].dx;
+            int dy = leaper_description[nn].dy;
+            bitboard_t<kind> bb = movegen.make_leaper_bitboard(square, dx, dy);
+
+            int count = (bb & leaper).popcount();
+            if (count > best_count) {
+               best_count = count;
+               best = nn;
+               best_leaper = bb;
+
+               if (leaper == bb) break;
             }
          }
 
-         /* Construct symmetry masks */
-         if (is_aleaper(flags)) {
-            bitboard_t<kind> fd, bd, fa, ba;
-            int rank = unpack_rank(square);
-            int file = unpack_file(square);
-            int diag = bitboard_t<kind>::diagonal_nr[square];
-            int anti = bitboard_t<kind>::anti_diagonal_nr[square];
+         if (best == -1) break;  /* Break out if nothing matches */
 
-            sym4[F] = bitboard_t<kind>::board_northward[rank];
-            sym4[B] = bitboard_t<kind>::board_southward[rank];
-            sym4[R] = bitboard_t<kind>::board_eastward[file];
-            sym4[L] = bitboard_t<kind>::board_westward[file];
-            sym4[V] = sym4[F] | sym4[B];
-            sym4[S] = sym4[R] | sym4[L];
+         if ((leaper & best_leaper) == best_leaper) {
+            n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
+         } else {
+            assert(is_aleaper(flags));
 
-            sym4[RF4] = sym4[R] & sym4[L];
-            sym4[LF4] = sym4[L] & sym4[L];
-            sym4[RB4] = sym4[R] & sym4[B];
-            sym4[LB4] = sym4[L] & sym4[B];
+            /* Only some of the directions match, find out which */
+            if (leaper_description[best].symmetry == 4) { /* 4-point symmetry */
+               while (!(leaper & best_leaper).is_empty()) {
+                  int best = -1;
+                  int best_count = 0;
+                  for (int k = 0; k<NUM_SYM4; k++) {
+                     /* Make sure the direction does not imply unwanted extra bits */
+                     if (!((sym4[k]&best_leaper) & ~(best_leaper&leaper)).is_empty()) continue;
 
-            sym8[FH] = sym4[F];
-            sym8[BH] = sym4[B];
-            sym8[LH] = sym4[L];
-            sym8[RH] = sym4[R];
+                     bitboard_t<kind> bb = sym4[k] & best_leaper & leaper;
 
-            fd.clear();
-            for (int n = diag+1; n<16; n++)
-               fd |= bitboard_t<kind>::board_diagonal[n];
-            fa.clear();
-            for (int n = anti+1; n<16; n++)
-               fa |= bitboard_t<kind>::board_antidiagonal[n];
-            bd.clear();
-            for (int n = 0; n < diag; n++)
-               bd |= bitboard_t<kind>::board_diagonal[n];
-            ba.clear();
-            for (int n = 0; n < anti; n++)
-               ba |= bitboard_t<kind>::board_antidiagonal[n];
+                     int count = bb.popcount();
+                     if (count > best_count) {
+                        best_count = count;
+                        best = k;
+                     }
+                  }
 
-            sym8[FF] = fd & fa;
-            sym8[BB] = bd & ba;
-            sym8[LL] = fd & ba;
-            sym8[RR] = bd & fa;
+                  if (best == -1) break;  /* Break out if nothing matches */
 
-            sym8[FL] = sym8[FH] & sym8[LL];
-            sym8[LF] = sym8[LH] & sym8[FF];
-            sym8[FR] = sym8[FH] & sym8[RR];
-            sym8[RF] = sym8[RH] & sym8[FF];
-            sym8[BL] = sym8[BH] & sym8[LL];
-            sym8[LB] = sym8[LH] & sym8[BB];
-            sym8[BR] = sym8[BH] & sym8[RR];
-            sym8[RB] = sym8[RH] & sym8[BB];
-         }
+                  n += snprintf(buffer+n, size-n, "%s", sym4txt[best]);
 
-         /* Find which combination of simple leapers matches best.
-          * We try the non-atom king first.
-          */
-         bitboard_t<kind> kb = movegen.make_leaper_bitboard(square, 1, 1)|movegen.make_leaper_bitboard(square, 1, 0);
-         if (leaper == kb) {
-            n += snprintf(buffer+n, size-n, "K");
-            leaper.clear();
-         }
+                  leaper &= ~(best_leaper & sym4[best]);
+               }
+            } else if (leaper_description[best].symmetry == 8) { /* 8-point symmetry */
+               while (!(leaper & best_leaper).is_empty()) {
+                  int best = -1;
+                  int best_count = 0;
+                  for (int k = 0; k<NUM_SYM8; k++) {
+                     /* Make sure the direction does not imply unwanted extra bits */
+                     if (!((sym8[k]&best_leaper) & ~(best_leaper&leaper)).is_empty()) continue;
 
-         /* Loop over all fundamental leapers.
-          * FIXME: the loop can become infinite if the leaper is long (> 4) in any direction.
-          */
-         while (!leaper.is_empty()) {
-            bitboard_t<kind> best_leaper;
-            int best = -1;
-            int best_count = 0;
-            for (int nn = 0; leaper_description[nn].txt; nn++) {
-               int dx = leaper_description[nn].dx;
-               int dy = leaper_description[nn].dy;
-               bitboard_t<kind> bb = movegen.make_leaper_bitboard(square, dx, dy);
+                     bitboard_t<kind> bb = sym8[k] & best_leaper & leaper;
 
-               int count = (bb & leaper).popcount();
-               if (count > best_count) {
-                  best_count = count;
-                  best = nn;
-                  best_leaper = bb;
+                     int count = bb.popcount();
+                     if (count > best_count) {
+                        best_count = count;
+                        best = k;
+                     }
+                  }
 
-                  if (leaper == bb) break;
+                  if (best == -1) break;  /* Break out if nothing matches */
+                  n += snprintf(buffer+n, size-n, "%s", sym8txt[best]);
+
+                  leaper &= ~(best_leaper & sym8[best]);
                }
             }
-
-            if (best == -1) break;  /* Break out if nothing matches */
-
-            if ((leaper & best_leaper) == best_leaper) {
-               n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
-            } else {
-               assert(is_aleaper(flags));
-
-               /* Only some of the directions match, find out which */
-               if (leaper_description[best].symmetry == 4) { /* 4-point symmetry */
-                  while (!(leaper & best_leaper).is_empty()) {
-                     int best = -1;
-                     int best_count = 0;
-                     for (int k = 0; k<NUM_SYM4; k++) {
-                        /* Make sure the direction does not imply unwanted extra bits */
-                        if (!((sym4[k]&best_leaper) & ~(best_leaper&leaper)).is_empty()) continue;
-
-                        bitboard_t<kind> bb = sym4[k] & best_leaper & leaper;
-
-                        int count = bb.popcount();
-                        if (count > best_count) {
-                           best_count = count;
-                           best = k;
-                        }
-                     }
-
-                     if (best == -1) break;  /* Break out if nothing matches */
-
-                     n += snprintf(buffer+n, size-n, "%s", sym4txt[best]);
-
-                     leaper &= ~(best_leaper & sym4[best]);
-                  }
-               } else if (leaper_description[best].symmetry == 8) { /* 8-point symmetry */
-                  while (!(leaper & best_leaper).is_empty()) {
-                     int best = -1;
-                     int best_count = 0;
-                     for (int k = 0; k<NUM_SYM8; k++) {
-                        /* Make sure the direction does not imply unwanted extra bits */
-                        if (!((sym8[k]&best_leaper) & ~(best_leaper&leaper)).is_empty()) continue;
-
-                        bitboard_t<kind> bb = sym8[k] & best_leaper & leaper;
-
-                        int count = bb.popcount();
-                        if (count > best_count) {
-                           best_count = count;
-                           best = k;
-                        }
-                     }
-
-                     if (best == -1) break;  /* Break out if nothing matches */
-                     n += snprintf(buffer+n, size-n, "%s", sym8txt[best]);
-
-                     leaper &= ~(best_leaper & sym8[best]);
-                  }
-               }
-               n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
-            }
-
-            leaper &= ~best_leaper;
+            n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
          }
+
+         leaper &= ~best_leaper;
       }
    }
 
@@ -614,6 +605,41 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
       /* Hack: try to convince XBoard that this piece can do anything */
       snprintf(buffer, size, "QNCZt");
    }
+
+   /* Drop extension,
+    * http://www.talkchess.com/forum/viewtopic.php?p=612101#612101
+    */
+   if (board.rule_flags & RF_USE_DROPS) {
+      bool print = false;
+      /* No drops on first file */
+      if ((pt.drop_zone[WHITE][piece] & bitboard_t<kind>::board_rank[0]).is_empty()) {
+         n += snprintf(buffer+n, size-n, "j");
+         print = true;
+      }
+      /* Restriction on number of pieces of this type, board */
+      if (pt.piece_maximum[piece][WHITE]<128 && !(pt.piece_maximum[piece][WHITE]==1 && (pt.piece_flags[piece]&PF_ROYAL))) {
+         print = true;
+         for (int k = 0; k<pt.piece_maximum[piece][WHITE]; k++)
+            n += snprintf(buffer+n, size-n, "b");
+      }
+      /* Restriction on number of pieces of this type, file */
+      if (pt.piece_flags[piece] & PF_DROPONEFILE) {
+         print = true;
+         for (int k = 0; k<pt.piece_drop_file_maximum[piece]; k++)
+            n += snprintf(buffer+n, size-n, "f");
+      }
+      /* Shorten promotion zone */
+      int last_rank = ranks;
+      while (last_rank > 0 && (pt.drop_zone[WHITE][piece] & bitboard_t<kind>::board_rank[last_rank-1]).is_empty())
+         last_rank--;
+      if (last_rank != ranks)
+         print = true;
+      if (print) {
+         n += snprintf(buffer+n, size-n, "@");
+         if (last_rank != ranks)
+            n += snprintf(buffer+n, size-n, "%d", last_rank);
+      }
+   }
    return buffer;
 }
 
