@@ -46,8 +46,9 @@
 #define RF_ALLOW_PICKUP       0x00000800  /* The player is allowed to take pieces in-hand */
 
 #define RF_PROMOTE_IN_PLACE   0x00001000  /* Promotions can be done in-place, without moving a piece */
+#define RF_PROMOTE_ON_DROP    0x00002000  /* Pieces can promote when they are dropped */
 
-#define RF_SPECIAL_IS_INIT    0x00002000  /* Special moves are only initial moves */
+#define RF_SPECIAL_IS_INIT    0x00004000  /* Special moves are only initial moves */
 
 #define RF_USE_HOLDINGS       (RF_USE_DROPS | RF_USE_CAPTURE | RF_ALLOW_PICKUP)  /* Game uses holdings in some way */
 
@@ -68,7 +69,7 @@ template <typename kind>
 struct unmake_info_t {
    bitboard_t<kind> init;
    bitboard_t<kind> ep;
-   uint64_t pawn_hash;
+   uint64_t board_hash;
    uint64_t hash;
    int8_t fifty_counter;
 
@@ -76,7 +77,7 @@ struct unmake_info_t {
 
    uint8_t board_flags;
 
-   piece_t pickup_piece[4];
+   uint8_t pickup_piece[4];
 #ifdef DEBUGMODE
    move_t move;
 #endif
@@ -101,7 +102,7 @@ struct board_t {
 
    /* Hash key */
    uint64_t hash;
-   uint64_t pawn_hash;
+   uint64_t board_hash;
 
    /* Rule flags, to change the behaviour of the move generator or the evaluation function */
    uint32_t rule_flags;
@@ -151,7 +152,7 @@ struct board_t {
       memset(holdings, 0, sizeof holdings);
 
       hash = 0;
-      pawn_hash = 0;
+      board_hash = 0;
 
       board_flags = 0;
 
@@ -170,7 +171,7 @@ struct board_t {
       if (piece_types->piece_flags[type] & PF_ROYAL)
          royal.set(square);
       hash ^= piece_key[type][side][square];
-      if (type == piece_types->pawn_index[side]) pawn_hash ^= piece_key[type][side][square];
+      board_hash ^= piece_key[type][side][square];
    }
 
    void clear_piece(int type, side_t side, int square)
@@ -180,7 +181,7 @@ struct board_t {
       royal.reset(square);
       init.reset(square);
       hash ^= piece_key[type][side][square];
-      if (type == piece_types->pawn_index[side]) pawn_hash ^= piece_key[type][side][square];
+      board_hash ^= piece_key[type][side][square];
    }
 
    void put_new_piece(int type, side_t side, int square)
@@ -189,7 +190,11 @@ struct board_t {
       init.set(square);
    }
 
-   int8_t get_piece(int square) const { return piece[square]; }
+   int8_t get_piece(int square) const {
+      assert(square >= 0);
+      assert(square < 8*sizeof(kind));
+      return piece[square];
+   }
    side_t get_side(int square) const {
       if (bbc[WHITE].test(square)) return WHITE;
       if (bbc[BLACK].test(square)) return BLACK;
@@ -222,6 +227,7 @@ struct board_t {
       /* First: backup information for unmake */
       ui->init = init;
       ui->hash = hash;
+      ui->board_hash = board_hash;
       ui->fifty_counter = fifty_counter;
       ui->ep = ep;
       ui->ep_victim = ep_victim;
@@ -292,6 +298,7 @@ struct board_t {
       if (expect((move & MOVE_KEEP_TURN) == 0, true)) {
          side_to_move = next_side[side_to_move];
          hash ^= side_to_move_key;
+         board_hash ^= side_to_move_key;
       }
 
       /* Finally: update 50-move clock */
@@ -363,6 +370,7 @@ struct board_t {
       /* Finally: restore backedup information */
       init = ui->init;
       hash = ui->hash;
+      board_hash = ui->board_hash;
       fifty_counter = ui->fifty_counter;
       ep = ui->ep;
       ep_victim = ui->ep_victim;
