@@ -48,7 +48,7 @@ void game_template_t<kind>::calculate_pawn_structure(pawn_structure_t<kind> *ps)
       bitboard_t<kind> bb = pawns &  board.bbc[side];
       bitboard_t<kind> ob = pawns & ~board.bbc[side];
 
-      /* Shelter score */
+      /* Shelter score (0-8) */
       for (int f = 0; f<bitboard_t<kind>::board_files; f++) {
          int lf = (f == 0)       ? f+1 : f-1;
          int rf = (f == files-1) ? f-1 : f+1;
@@ -114,6 +114,7 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
    int num_pieces[NUM_SIDES]     = { 0, 0 };
    int num_royals[NUM_SIDES]     = { 0, 0 };
    int num_pawns[NUM_SIDES]      = { 0, 0 };
+   int num_supers[NUM_SIDES]      = { 0, 0 };
    int num_def[NUM_SIDES]        = { 0, 0 };
    int num_light_bound[NUM_SIDES]= { 0, 0 };
    int num_dark_bound[NUM_SIDES] = { 0, 0 };
@@ -164,6 +165,9 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             gate_space += board.holdings[n][side];
       }
 
+      /* Space advantage, * http://www.talkchess.com/forum/viewtopic.php?p=609260 */
+      psq.mg += PST_SPACE_MG*(bitboard_t<kind>::board_homeland[side] & ~occ).popcount();
+
       for (int n=0; n<pt.num_piece_types; n++) {
          int piece = perm[n];
          bitboard_t<kind> bb = board.bbc[side] & board.bbp[piece];
@@ -263,6 +267,9 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             if (pt.minor_pieces & (1<<piece))
                minors[side].set(square);
 
+            if (pt.super_pieces & (1<<piece))
+               num_supers[side]++;
+
             if (pt.pawn_pieces & (1<<piece)) {
                pawn_attacks[side] |= atk;
                pawns[side].set(square);
@@ -293,6 +300,13 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
       psq = -psq;
    }
 
+   /* Take into account defensive material */
+   if (mat.mg < 0) mat.mg = std::min(0, mat.mg + def[WHITE].mg/2);
+   if (mat.eg < 0) mat.eg = std::min(0, mat.eg + def[WHITE].eg/2);
+   if (mat.mg > 0) mat.mg = std::max(0, mat.mg - def[BLACK].mg/2);
+   if (mat.eg > 0) mat.eg = std::max(0, mat.eg - def[BLACK].eg/2);
+
+   /* Gather data for shelter and assess winning chances. */
    for (side_t side = WHITE; side<NUM_SIDES; side++) {
       side_t oside = next_side[side];
 
@@ -305,8 +319,8 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
          eval_t cscore = 0;
          int f = unpack_file(king[side]);
 
-         score = ps.shelter_score[side][f];
-         shelter[side] = 4*score;
+         score = ps.shelter_score[side][f];  /* 0-8 */
+         shelter[side] = 4*score;            /* 0-32 */
 
          if ((board.init & movegen.castle_mask[SHORT][side] & board.bbc[side]) == movegen.castle_mask[SHORT][side]) {
             int square = movegen.castle_king_dest[SHORT][side];
@@ -320,7 +334,6 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             cscore = std::max(cscore, ps.shelter_score[side][f]);
          }
 
-         //(board.init & movegen.castle_mask[LONG][side] & board.bbc[side]).print();
          //printf("Shelter: %d %d\n", score, cscore);
          if (cscore > score) score = (score + cscore) / 2;
 
@@ -452,18 +465,11 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
       /* Flip evaluation values for side to move.
        * Idea from Senpai.
        */
-      mat = -mat;
       psq = -psq;
       kss = -kss;
       pss = -pss;
    }
 
-   /* Take into account defensive material */
-   if (mat.mg < 0) mat.mg = std::min(0, mat.mg + def[WHITE].mg/2);
-   if (mat.eg < 0) mat.eg = std::min(0, mat.eg + def[WHITE].eg/2);
-   if (mat.mg > 0) mat.mg = std::max(0, mat.mg - def[BLACK].mg/2);
-   if (mat.eg > 0) mat.eg = std::max(0, mat.eg - def[BLACK].eg/2);
-
    /* Draw-ish material combinations */
    if (num_pawns[WHITE] == 0 && num_pawns[BLACK] == 0) {
       if (mate_potential[WHITE] == mate_potential[BLACK] && mate_potential[WHITE] == 1 && abs(num_pieces[WHITE]-num_pieces[BLACK]) <= 1)
@@ -560,19 +566,20 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             if (pt.piece_flags[piece] & PF_ROYAL) {
                bitboard_t<kind> safe   = moves[square] & ~all_attacks[oside];
                bitboard_t<kind> unsafe = moves[square] ^ safe;
+
                mob.eg += pt.eval_mobility[piece][safe.popcount()].eg   * SAFE_MOB_WEIGHT / 128;
             } else {
                bitboard_t<kind> up_attacks = all_attacks[oside] ^ less_attacks[oside][piece];
-               bitboard_t<kind> safe = moves[square] & ~less_attacks[oside][piece];
-               safe |= moves[square] & up_attacks & all_attacks[side];
-               bitboard_t<kind> unsafe = moves[square] ^ safe;
+               bitboard_t<kind> safe       = (moves[square] & ~less_attacks[oside][piece]) |
+                                             (moves[square] & up_attacks & all_attacks[side]);
+               bitboard_t<kind> unsafe     = moves[square] ^ safe;
 
-               mob += pt.eval_mobility[piece][safe.popcount()]   * SAFE_MOB_WEIGHT / 128;
+               mob    += pt.eval_mobility[piece][safe.popcount()]      * SAFE_MOB_WEIGHT / 128;
 
                bitboard_t<kind> forward = (side == WHITE) ?  bitboard_t<kind>::board_northward[unpack_rank(square)]
                                                           :  bitboard_t<kind>::board_southward[unpack_rank(square)];
                if ((moves[square] & forward & ~pawns[side]).is_empty())
-                  psq.mg -= 5;
+                  psq.mg -= MOB_FORWARD_BLOCKED;
             }
 
             /* Board control
@@ -599,6 +606,7 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             if (pt.minor_pieces & (1<<piece)) {
                /* Unprotected minor */
 
+               if (!pawn_attacks[side].test(square)) psq.mg -= ((1+num_supers[oside])*LOOSE_MINOR_PENALTY)/2;
                //if (less_attacks[side][piece].test(square)) psq.mg -= 10;
                //if (less_attacks[oside][piece].test(square)) psq.mg -= pt.piece_value[piece]/16;
                //else if (!all_attacks[oside].test(square)) psq.mg -= pt.piece_value[piece]/32;
@@ -681,6 +689,13 @@ done:
       printf("Pawn structure:  % 4d   % 4d\n", pss.mg, pss.eg);
       printf("King safety:     % 4d   % 4d\n", kss.mg, kss.eg);
    }
+
+   /* Disable phase-scaling in variants where captured pieces are returned:
+    * there material does not represent game phase.
+    */
+   if (board.rule_flags & RF_USE_CAPTURE) phase = pt.phase_scale;
+
+   /* Interpolate evaluation score. */
    ev = (mat + psq + mob + pss + kss).interpolate(phase, pt.phase_scale);
 
    /* Adjust the score: if the side that is nominally ahead can't win, drop the score to 0(ish) */
