@@ -4,24 +4,43 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
    int board_files = bitboard_t<kind>::board_files;
    int board_ranks = bitboard_t<kind>::board_ranks;
 
-   switch(state) {
-      case '-':
-         return;
+   int rook_file = -1;
+   int rook_from = -1;
 
+   if (state == '-') return;
+   if (!isalpha(state)) return;
+
+   side_t side = BLACK;
+   if (isupper(state)) side = WHITE;
+
+   char file_char = (side == WHITE) ? 'A' : 'a';
+
+   if (state-file_char < board_files) {
+      rook_file = state - file_char;
+      rook_from = bitboard_t<kind>::pack_rank_file( (side == WHITE) ? 0 : (board_ranks-1), rook_file);
+
+      castle_init->set(rook_from);
+
+      if (board.rule_flags & RF_GATE_DROPS) return;
+   }
+
+   switch(state) {
       case 'K':
          /* Starting at the right edge of the board, find the first castle
           * piece.
           */
          state = '-';
-         for (int n = bitboard_t<kind>::board_files - 1; n>=0; n--) {
-            if (board.get_piece(n) == pt.castle_piece[WHITE]) {
-               state = 'A' + unpack_file(n);
+         for (int n = board_files - 1; n>=0; n--) {
+            if (board.get_piece(n) == pt.castle_piece[side][SHORT]) {
+               rook_from = n;
+               state = file_char + n;
                break;
             }
 
          }
+         if (state != 'K')
          record_castle_state(state, castle_init);
-         return;
+         break;
 
       case 'Q':
          /* Starting at the left edge of the board, find the first castle
@@ -29,14 +48,16 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
           */
          state = '-';
          for (int n = 0; n<board_files; n++) {
-            if (board.get_piece(n) == pt.castle_piece[WHITE]) {
-               state = 'A' + unpack_file(n);
+            if (board.get_piece(n) == pt.castle_piece[side][LONG]) {
+               rook_from = n;
+               state = file_char + n;
                break;
             }
 
          }
+         if (state != 'Q')
          record_castle_state(state, castle_init);
-         return;
+         break;
 
       case 'k':
          /* Starting at the right edge of the board, find the first castle
@@ -45,14 +66,16 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
          state = '-';
          for (int n = board_files - 1; n>=0; n--) {
             int s = board_files * (board_ranks-1);
-            if (board.get_piece(s + n) == pt.castle_piece[BLACK]) {
-               state = 'a' + unpack_file(n);
+            if (board.get_piece(s + n) == pt.castle_piece[side][SHORT]) {
+               rook_from = s+n;
+               state = file_char + n;
                break;
             }
 
          }
+         if (state != 'k')
          record_castle_state(state, castle_init);
-         return;
+         break;
 
       case 'q':
          /* Starting at the left edge of the board, find the first castle
@@ -61,47 +84,49 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
          state = '-';
          for (int n = 0; n<board_files; n++) {
             int s = board_files * (board_ranks-1);
-            if (board.get_piece(s + n) == pt.castle_piece[BLACK]) {
-               state = 'a' + unpack_file(n);
+            if (board.get_piece(s + n) == pt.castle_piece[side][LONG]) {
+               rook_from = s+n;
+               state = file_char + n;
                break;
             }
 
          }
+         if (state != 'q')
          record_castle_state(state, castle_init);
-         return;
+         break;
    }
 
+   if (rook_from < 0) return;
+
    /* Fischer-random style castling. This identifies the file of the piece with which
     * we can castle. We can find the file of the king by looking at the royal bitboard, and we
     * know the king destination after castling from the rule description. From this, we can work
     * out the required bitmasks.
     */
-   side_t side = BLACK;
-   if (isupper(state)) side = WHITE;
-   int king_from, king_to, rook_from;
-   int rook_file;
-   king_from = (board.royal & board.bbc[side]).bitscan();
-   rook_file = tolower(state) - 'a';
-   rook_from = bitboard_t<kind>::pack_rank_file( (side == WHITE) ? 0 : (board_ranks-1), rook_file);
-   king_to = movegen.castle_king_dest[LONG][side];
-   if (rook_from > king_from) king_to = movegen.castle_king_dest[SHORT][side];
+   int king_from = -1, king_to;
+
+   for (int n = 0; n<pt.num_piece_types; n++) {
+      if (pt.piece_flags[n] & PF_CASTLE) {
+         king_from = (board.bbc[side] & board.bbp[n]).bitscan();
+         break;
+      }
+   }
+   if (king_from < 0) return;
+   int castle_side = LONG;
+   if (unpack_file(rook_from) >= unpack_file(king_from)) castle_side = SHORT;
+
+   king_to = movegen.castle_king_dest[castle_side][side];
 
    //printf("%d (%s %s) (%s)\n", side, square_names[king_from], square_names[king_to], square_names[rook_from]);
    movegen.deduce_castle_flags(side, king_from, king_to, rook_from);
 
    castle_init->set(king_from);
-   castle_init->set(rook_from);
-
-   /* Clear castle flags by default */
-   board.init.reset(rook_from);
-   board.init.reset(king_from);
 
    return;
 }
 
-void setup_fen_position(const char *str)
+void setup_fen_position(const char *str, bool skip_castle = false)
 {
-   char pname[5];
    const char *s = str;
    int prev_rank = 2*bitboard_t<kind>::board_files;
    int square = top_left;
@@ -110,7 +135,8 @@ void setup_fen_position(const char *str)
    if (!s) return;
    moves_played = 0;
    board.clear();
-   memset(repetition_hash_table, 0, sizeof(repetition_hash_table));
+   memset(repetition_hash_table, 0, sizeof repetition_hash_table);
+   memset(board_repetition_hash_table, 0, sizeof board_repetition_hash_table);
 
    /* Parse first record: piece positions */
    while(*s && (*s != ' ') && (*s != '[') && square>=0) {
@@ -139,25 +165,33 @@ void setup_fen_position(const char *str)
             break;
 
          default:
-            memset(pname, 0, sizeof pname);
-            pname[0] = *s;
-            if (*s == '+') { s++; pname[1] = *s; }
+            side_t side = NONE;
+            int piece = -1;
+            int best_len = 0;
             for (n=0; n<pt.num_piece_types; n++) {
                /* Determine piece colour */
-               side_t side = NONE;
-               if (strstr(pt.piece_abbreviation[n][WHITE], pname)) side = WHITE;
-               if (strstr(pt.piece_abbreviation[n][BLACK], pname)) side = BLACK;
-               if (side != NONE) {
-                  assert(square_to_bit[square] >= 0);
-                  board.put_new_piece(n, side, square_to_bit[square]);
-                  square++;
-                  break;
+               side_t match_side = NONE;
+               if (strstr(s, pt.piece_abbreviation[n][WHITE]) == s) match_side = WHITE;
+               if (strstr(s, pt.piece_abbreviation[n][BLACK]) == s) match_side = BLACK;
+
+               if (match_side != NONE) {
+                  int len = strlen(pt.piece_abbreviation[n][match_side]);
+                  if (len > best_len) {
+                     best_len = len;
+                     side = match_side;
+                     piece = n;
+                  }
                }
             }
-            if (n == pt.num_piece_types) {
-               if (error_output) error_output("Error: unknown piece type '%s' (bad FEN %s)\n", pname, str);
+            if (piece == -1) {
+               if (error_output) error_output("Error: unknown piece type '%c' (bad FEN %s)\n", *s, str);
                return;
             }
+            assert(side != NONE);
+            assert(square_to_bit[square] >= 0);
+            board.put_new_piece(piece, side, square_to_bit[square]);
+            s += strlen(pt.piece_abbreviation[piece][side])-1;
+            square++;
             break;
       }
       s++;
@@ -195,26 +229,26 @@ void setup_fen_position(const char *str)
    /* Initialise attack map */
    //initialise_attack_tables(&board);
 
-   /* Third record: castling rights.
+   /* Third record: castling rights/init state.
     * Skip if the next entry is a number, in which case the game doesn't have castling.
     */
    while(*s && (*s == ' ')) s++;
    if (!isdigit(*s)) {
-      bitboard_t<kind> castle_init;
 
       /* Clear castle flags by default */
-      board.init &= ~board.royal;
+      for (int n = 0; n<pt.num_piece_types; n++) {
+         if (!(pt.pawn_pieces & (1<<n)))
+            board.init &= ~board.bbp[n];
+      }
 
       while(*s && (*s != ' ')) {
-         record_castle_state(*s, &castle_init);
+         bitboard_t<kind> castle_init;
+         if (!skip_castle)
+            record_castle_state(*s, &castle_init);
          s++;
+         board.init |= castle_init;
       }
       if (*s) s++;
-
-      /* Set castle flags.
-       * Do a paranoid sanity check to make sure that only occupied squares have their init-bits set.
-       */
-      board.init |= castle_init;
    }
 
    /* Make sure the initial flags are at least somewhat sane by making sure only occupied squares have their
@@ -247,6 +281,7 @@ void setup_fen_position(const char *str)
                board.ep_victim = board.ep.bitscan() + bitboard_t<kind>::board_files;
          }
       }
+      while(*s && (*s != ' ')) s++;
    }
 
    /* Fifth record: half-move counter (50 move counter) */ 
@@ -256,8 +291,15 @@ void setup_fen_position(const char *str)
    board.fifty_counter = n;
    while(*s && (*s != ' ')) s++;
 
-   memset(repetition_hash_table, 0, sizeof repetition_hash_table);
+   /* Sixth record: full-move counter */
+   while(*s && (*s == ' ')) s++;
+   n = 0;
+   sscanf(s, "%d\n", &n);
+   start_move_count = 2*std::max(0, n-1);
+   while(*s && (*s != ' ')) s++;
+
    repetition_hash_table[board.hash&0xFFFF] = 1;
+   board_repetition_hash_table[board.board_hash&0xFFFF] = 1;
 
    /* Record check state */
    board.check(player_in_check(board.side_to_move));
@@ -311,8 +353,6 @@ const char *make_fen_string(char *buffer = NULL) const
       if (r) n += snprintf(fen+n, 4096 - n, "/");
    }
 
-   n += snprintf(fen+n, 4096 - n, " ");
-
    /* Holdings */
    if (board.rule_flags & RF_USE_HOLDINGS) {
       n += snprintf(fen+n, 4096 - n, "[");
@@ -326,9 +366,11 @@ const char *make_fen_string(char *buffer = NULL) const
          }
       }
       if (empty) n += snprintf(fen+n, 4096 - n, "-");
-      n += snprintf(fen+n, 4096 - n, "] ");
+      n += snprintf(fen+n, 4096 - n, "]");
    }
 
+   n += snprintf(fen+n, 4096 - n, " ");
+
    /* Second record: side to move */
    if (board.side_to_move == WHITE)
       n += snprintf(fen+n, 4096 - n, "w ");
@@ -371,16 +413,20 @@ const char *make_fen_string(char *buffer = NULL) const
    }
 
    /* Fourth record: en-passant square */
-   if (!board.ep.is_empty())
-      n += snprintf(fen+n, 4096 - n, "%s", square_names[board.ep.bitscan()]);
-   else
-      n += snprintf(fen+n, 4096 - n, "-");
-   n += snprintf(fen+n, 4096 - n, " ");
+   bool have_ep = false;
+   for (int n = 0; n<pt.num_piece_types; n++)
+      have_ep = have_ep || (pt.piece_flags[n] & PF_SET_EP);
+   if (have_ep) {
+      if (!board.ep.is_empty())
+         n += snprintf(fen+n, 4096 - n, "%s ", square_names[board.ep.bitscan()]);
+      else
+         n += snprintf(fen+n, 4096 - n, "- ");
+   }
 
    /* Fifth and sixth record: half-move counter and full-move counter */
    int par = (moves_played&1) && next_side[board.side_to_move];
    n += snprintf(fen+n, 4096 - n, "%d ", board.fifty_counter);
-   n += snprintf(fen+n, 4096 - n, "%d", (int)(moves_played + par)/2 + 1);
+   n += snprintf(fen+n, 4096 - n, "%d", (int)(start_move_count + moves_played + par)/2 + 1);
 
    return fen;
 }
