@@ -20,12 +20,15 @@
 #define uci  if (uci_output)       uci_output
 #define xb   if (xboard_output)    xboard_output
 
-static const int clock_nodes = 0x00007FFF;
 #define check_clock() \
+{\
    if (clock.max_nodes && (clock.nodes_searched >= clock.max_nodes)) abort_search |= 1;\
    if (check_keyboard && check_keyboard(this)) abort_search |= 1;\
    if (clock.check_clock && ((clock.nodes_searched & clock_nodes)==0))\
-      abort_search |= clock.check_clock(&clock)
+      abort_search |= clock.check_clock(&clock);\
+}(void)0
+
+#include "mate.h"
 
 void dump_moves_since_root(void)
 {
@@ -148,8 +151,7 @@ inline bool position_repeated()
 {
    int rep_in_game = 0;
    int count = 0;
-   if (repetition_hash_table[board.hash&0xFFFF]>=2)
-   {
+   if (repetition_hash_table[board.hash&0xFFFF]>=2) {
       int n;
       for (n=(int)moves_played-2; n>=0; n-=2) {
          if ((board.rule_flags & RF_USE_CAPTURE) == 0 && is_irreversible_move(move_list[n])) break;
@@ -163,8 +165,45 @@ inline bool position_repeated()
       }
    }
 
-   return count != 0;
-   return count >= (1 + rep_in_game);
+   return count && count >= rep_in_game;
+}
+
+inline bool board_position_repeated_loss()
+{
+   if (board_repetition_hash_table[board.board_hash&0xFFFF]>=2) {
+      const side_t me = board.side_to_move;
+      bool pure_loss = true;
+      bool pure_gain = true;
+      int8_t holdings[MAX_PIECE_TYPES][NUM_SIDES];
+      memset(holdings, 0, sizeof holdings);
+
+      for (int n=(int)moves_played-1; n>=0; n--) {
+         move_t move = move_list[n];
+
+         if ((board.rule_flags & RF_USE_CAPTURE) == 0 && is_irreversible_move(move)) break;
+
+         if (get_move_holdings(move)) {
+            uint16_t p  = get_move_holding(move);
+            int count   = decode_holding_count(p);
+            int piece   = decode_holding_piece(p);
+            side_t side = decode_holding_side(p);
+
+            holdings[piece][side] += count;
+         }
+
+         if (ui[n].board_hash == board.board_hash && ui[n].hash != board.hash) {
+            for (int piece = 0; piece<pt.num_piece_types; piece++) {
+               if (holdings[piece][me] < 0) pure_gain = false;
+               if (holdings[piece][me] > 0) pure_loss = false;
+            }
+
+            if (pure_gain) return false;
+            return pure_loss;
+         }
+      }
+   }
+
+   return false;
 }
 
 inline bool material_draw()
@@ -207,10 +246,9 @@ inline bool zugzwang_threat()
    return tempo_movers <= 1;
 }
 
-bool is_pawn_push(move_t move)
+bool is_pawn_push(move_t move, side_t side)
 {
    if (pt.pawn_pieces & (1 << get_move_piece(move))) {
-      int side = get_move_player(move);
       if (bitboard_t<kind>::board_homeland[next_side[side]].test(get_move_from(move)))
          return true;
    }
@@ -224,6 +262,101 @@ bool is_recapture_move(move_t move, move_t prev_move)
           get_move_to(move) == get_move_to(prev_move);
 }
 
+void score_moves(int depth, move_t hash_move, move_t prev_move, move_t threat_move)
+{
+   const side_t me = board.side_to_move;
+
+   for (int n = 0; n<movelist[depth].num_moves; n++) {
+      move_t move = movelist[depth].move[n];
+      movelist[depth].score[n] = 0;
+      if (move == hash_move) {
+         movelist[depth].score[n] = LEGALWIN;
+         if (best_move[depth] == 0)
+            best_move[depth] = hash_move;
+      } else if (is_promotion_move(move)) {
+         movelist[depth].score[n] += 2200 + pt.piece_value[get_move_promotion_piece(move)] / 10;
+      } else if (mate_killer[depth] == move) {
+         movelist[depth].score[n] += 2100;
+      } else if (mate_killer[depth+2] == move) {
+         movelist[depth].score[n] += 2050;
+      } else if (is_killer(depth, move)) {
+         movelist[depth].score[n] += 2000;
+         if (move == killer[depth][0])
+            movelist[depth].score[n] += 50;
+      } else if (depth > 2 && is_killer(depth-2, move)) {
+         movelist[depth].score[n] += 900;
+         if (move == killer[depth-2][0])
+            movelist[depth].score[n] += 50;
+      } else if (is_killer(depth+2, move)) {
+         movelist[depth].score[n] += 800;
+         if (move == killer[depth+2][0])
+            movelist[depth].score[n] += 50;
+      } else if (is_counter(depth, prev_move, move)) {
+         movelist[depth].score[n] += 1500;
+      } else if (is_castle_move(move)) {
+         movelist[depth].score[n] += 600;
+      } else if (is_capture_move(move)) {
+         int s = see(move);
+         if (s > 0)
+            movelist[depth].score[n] += 2400 + s / 10;
+         else
+            movelist[depth].score[n] += s;
+
+         if (threat_move && get_move_to(move) == get_move_from(threat_move) && s < 0)
+            movelist[depth].score[n] += 1000;
+      } else if (is_pickup_move(move)) {
+         if (threat_move && get_move_to(threat_move) == get_move_from(move)) {
+            movelist[depth].score[n] += 1500;
+         } else {
+            movelist[depth].score[n] += 0;
+         }
+      } else if (is_drop_move(move)) {
+         int history_score = get_move_history_score(move);
+         int history_scale = get_move_history_scale(move);
+         int s = see(move);
+         if (s > 0) {
+            int h = history_scale ? 100 * history_score / history_scale : 100;
+            if (!(board.bbc[me] & board.royal & bitboard_t<kind>::neighbour_board[get_move_to(move)]).is_empty())
+               movelist[depth].score[n] += 1200 + h;
+            else if (bitboard_t<kind>::board_homeland[next_side[me]].test(get_move_to(move)))
+               movelist[depth].score[n] += 1100 + h;
+            movelist[depth].score[n] += s / 10;
+         } else {
+            int h = history_scale ? 500 * history_score / history_scale : 500;
+            movelist[depth].score[n] += s + h;
+         }
+      } else if (max_history && !board.check() && !is_promotion_move(move) && !is_capture_move(move)) {
+         int history_score = get_move_history_score(move);
+         int history_scale = get_move_history_scale(move);
+         if (history_scale) {
+            int s = 500 * history_score / history_scale;
+            if (s) {
+               movelist[depth].score[n] = 0 + s;
+            } else {
+               s = see(move);
+               movelist[depth].score[n] += s;
+            }
+         }
+
+         if (threat_move && get_move_from(move) == get_move_to(threat_move))
+            movelist[depth].score[n] += 500;
+      } else {
+         int s = see(move);
+         movelist[depth].score[n] += s;
+
+         if (threat_move && get_move_from(move) == get_move_to(threat_move))
+            movelist[depth].score[n] += 500;
+      }
+
+      if (is_gate_move(move))
+         movelist[depth].score[n] += 200;
+
+      if (depth < 3 && clock.root_moves_played < 10 && random_ok) {
+         movelist[depth].score[n] += genrandui() & 0xff;
+      }
+   }
+}
+
 int qsearch(int alpha, int beta, int draft, int depth)
 {
    int score = -LEGALWIN;
@@ -235,10 +368,6 @@ int qsearch(int alpha, int beta, int draft, int depth)
    check_clock();
    if (abort_search) return 0;
 
-   if (!board.check() && draft < -6 && beta == alpha+1) {
-      return static_qsearch(beta, depth+1);
-   }
-
    if (!board.check() && (board.rule_flags & RF_FORCE_CAPTURE) && (draft < -2)) {
       return static_qsearch(beta, depth+1);
    }
@@ -263,7 +392,7 @@ int qsearch(int alpha, int beta, int draft, int depth)
 
    /* Generate moves */
    movelist[depth].clear();
-   movegen.generate_moves(&movelist[depth], &board, me, true);
+   movegen.generate_moves(&movelist[depth], &board, me, true, pt.deferral_allowed);
 
    if (movelist[depth].num_moves == 0)
       return static_score;
@@ -288,6 +417,7 @@ int qsearch(int alpha, int beta, int draft, int depth)
     * Explicitly cut quiet moves when not in check; the move generator
     * still produces some of these if promotions are optional.
     */
+   int legal_moves = movelist[depth].num_moves;
    int best_score = score;
    int best_score2 = best_score;
    move_t move;
@@ -301,6 +431,7 @@ int qsearch(int alpha, int beta, int draft, int depth)
       if (is_pickup_move(move)) continue;
       playmove(move);
       if (player_in_check(me)) {
+         legal_moves--;
          takeback();
          continue;
       }
@@ -323,6 +454,44 @@ int qsearch(int alpha, int beta, int draft, int depth)
    if (board.rule_flags & RF_ALLOW_PICKUP) best_score = best_score2;
    if (best_score == -LEGALWIN) best_score = static_score;
 
+   if (board.check() && legal_moves == 0) {
+      best_score = (mate_score + depth);
+
+      move_t prev_move = move_list[moves_played-1];
+
+      /* Make sure mate wasn't caused by an illegal drop */
+      if (board.check() && is_drop_move(prev_move)) {
+         int p = get_move_piece(prev_move);
+         if (pt.piece_flags[p] & PF_DROPNOMATE)
+            return ILLEGAL;
+      }
+
+      /* Test if the mate is legal, that is to say:
+       *  - The checking piece is allowed to deliver mate
+       *  - The mate follows a sequence of checks at least one of which
+       *    makes the mate OK (is classified as "shak").
+       * The first is illegal, the second is only a draw.
+       */
+      if (board.check()) {
+         if (!is_valid_mate())
+            return ILLEGAL;
+
+         if (board.rule_flags & RF_USE_SHAKMATE) {
+            if (!board.have_shak()) best_score = 0;
+         }
+      }
+
+      /* Verify that all moves during QS were check moves, so we know that
+       * all evasions were examined.
+       */
+      int c = 2;
+      for (int n=draft+2; n<=0; n+=2) {
+         if ((ui[moves_played-c].board_flags & BF_CHECK) == 0)
+            best_score = static_score;
+         c+=2;
+      }
+   }
+      
    return best_score;
 }
 
@@ -336,7 +505,7 @@ int static_qsearch(int beta, int depth)
       return static_score;
 
    movelist[depth].clear();
-   movegen.generate_moves(&movelist[depth], &board, board.side_to_move, true);
+   movegen.generate_moves(&movelist[depth], &board, board.side_to_move, true, pt.deferral_allowed);
 
    bitboard_t<kind> cdone;
    for (int n=0; n<movelist[depth].num_moves; n++) {
@@ -388,12 +557,10 @@ int search(int alpha, int beta, int draft, int depth)
 {
    assert(beta > alpha);
    assert(depth >= 0);
-   int max_see = -LEGALWIN;
    int best_score = -LEGALWIN;
    int score = -LEGALWIN;
    int moves_searched = 0;
-   side_t me = board.side_to_move;
-   move_t my_prev_move = 0;
+   const side_t me = board.side_to_move;
    move_t prev_move = 0;
    move_t move = 0;
 
@@ -448,19 +615,29 @@ int search(int alpha, int beta, int draft, int depth)
                   break;
             }
          } else {
+            if (board.check()) {
+               if (perpetual == LEGALDRAW)
+                  return -perpetual;
+
+               if (perpetual == -ILLEGAL)
+                  return -perpetual;
+
+               return -(perpetual + depth);
+            }
+
             if (rep_score == LEGALDRAW)
                return rep_score;
 
             /* Allow check evasions to repeat */
             if (rep_score == -ILLEGAL) {
-               //if (!board.check())
                if ((ui[moves_played-1].board_flags & BF_CHECK) == 0) /* HACK: allow check evasion */
-                  return ILLEGAL;
+                  return -rep_score;
             } else {
-               if (!board.check())
-                  return -(rep_score + depth);
+               return -(rep_score + depth);
             }
          }
+      } else if ((board.rule_flags & RF_USE_CAPTURE) && board_position_repeated_loss() && depth) {
+         return alpha-1;
       }
 
       if (material_draw())
@@ -473,11 +650,24 @@ int search(int alpha, int beta, int draft, int depth)
              * turn, so if the enemy could have captured our last piece he
              * clearly failed to do so - in other words, we have won.
              */
-            return LEGALWIN;
+            return -bare_king_score;
          }
       }
    }
 
+   //if ( (depth == 0 || depth == 1 || draft > 8 || is_mate_score(beta)) && (board.rule_flags & RF_USE_DROPS)) {
+   bool mate_search = (depth == 1) ||
+                      (depth == 0 && best_move[0] != 0) ||
+                      (draft > 8);  
+   if ( mate_search && ((option_ms >= MATE_SEARCH_ENABLED) || ((board.rule_flags & RF_USE_DROPS) && option_ms >= MATE_SEARCH_ENABLE_DROP)) ) {
+      int xply  = (depth < 2) ? draft / 4 : 0;
+      int mply  = (depth == 0 && best_move[0] == 0) ? 1 : (3 + 2*xply);
+      int score = msearch(alpha, beta, mply, depth);
+
+      if (is_mate_score(score))
+         return score;
+   }
+
    if (draft <= 0 || depth > MAX_SEARCH_DEPTH)
       return qsearch(alpha, beta, 0, depth);
 
@@ -486,7 +676,8 @@ int search(int alpha, int beta, int draft, int depth)
    unsigned int hash_flag;
    move_t hash_move = 0;
    bool have_hash = retrieve_table(transposition_table, board.hash, &hash_depth, &hash_score, &hash_flag, &hash_move);
-   if (have_hash && hash_depth >= draft && board.fifty_counter < 80 && depth > 0) {
+   bool hash_ok   = hash_depth >= draft || is_mate_score(hash_score);
+   if (have_hash && hash_ok && board.fifty_counter < 80 && depth > 0) {
       hash_score = score_from_hashtable(hash_score, depth);
       //positions_in_hashtable++;
       //bool exact_ok = (depth > 1) || (beta == alpha+1);
@@ -502,36 +693,44 @@ int search(int alpha, int beta, int draft, int depth)
       }
    }
 
+   /* Store the previous move: this is used to change move ordering on this
+    * move. If there is no previous move, then this move will be null.
+    */
+   if (moves_played > 0)
+      prev_move = move_list[moves_played-1];
+
+   /* Pre-conditions for particular pruning options. */
+   bool razor = draft <= 3 &&
+                !is_mate_score(beta) &&
+                !board.check();
+
+   bool avoid_null = beta > alpha+1 ||
+                     board.check() ||
+                     prev_move == 0 ||
+                     zugzwang_threat() ||
+                     is_mate_score(beta);
+
+   /* Calculate static evaluation if needed */
+   bool need_static = razor || !avoid_null;
    int static_score = 0;
-   if (!board.check())
+   if (need_static) {
       static_score = static_evaluation<false>(me, alpha, beta);
+      avoid_null = avoid_null || static_score < beta;
+   }
 
    /* Razoring */
-   if (beta == alpha-1 && draft <= 3 && !is_mate_score(beta) && !board.check()) {
+   if (razor) {
+      assert(need_static);
       int score = static_score - draft * draft * 50;
 
       if (score >= beta)
          return score;
    }
 
-
-   /* Store the previous move: this is used to change move ordering on this
-    * move. If there is no previous move, then this move will be null.
-    */
-   if (moves_played > 0) {
-      prev_move = move_list[moves_played-1];
-      if (moves_played > 1) my_prev_move = move_list[moves_played-2];
-   }
-
    /* Null-move */
    move_t threat_move = 0;
-   bool avoid_null = beta > alpha+1 ||
-                     board.check() ||
-                     static_score < beta ||
-                     prev_move == 0 ||
-                     zugzwang_threat() ||
-                     is_mate_score(beta);
    if (!avoid_null) {
+      assert(need_static);
       assert(!player_in_check(me));
       int r = 2;
       int score;
@@ -562,7 +761,7 @@ int search(int alpha, int beta, int draft, int depth)
 
    /* Generate all moves */
    movelist[depth].clear();
-   movegen.generate_moves(&movelist[depth], &board, me);
+   movegen.generate_moves(&movelist[depth], &board, me, false, pt.deferral_allowed);
    int legal_moves = movelist[depth].num_moves;
 
    /* No moves generated at all? */
@@ -570,106 +769,7 @@ int search(int alpha, int beta, int draft, int depth)
       goto no_moves;
 
    /* Score and sort the moves */
-   for (int n = 0; n<movelist[depth].num_moves; n++) {
-      move_t move = movelist[depth].move[n];
-      movelist[depth].score[n] = 0;
-      if (move == hash_move) {
-         movelist[depth].score[n] = LEGALWIN;
-         //if (is_capture_move(move) && draft < FUTILITY_DEPTH)
-         //   max_see = std::max(max_see, see(move));
-      } else if (is_promotion_move(move)) {
-         movelist[depth].score[n] += 2200 + pt.piece_value[get_move_promotion_piece(move)] / 10;
-      } else if (mate_killer[depth] == move) {
-         movelist[depth].score[n] += 2100;
-      } else if (is_killer(depth, move)) {
-         movelist[depth].score[n] += 2000;
-         if (move == killer[depth][0])
-            movelist[depth].score[n] += 50;
-      } else if (depth > 2 && is_killer(depth-2, move)) {
-         movelist[depth].score[n] += 900;
-         if (move == killer[depth-2][0])
-            movelist[depth].score[n] += 50;
-      } else if (is_killer(depth+2, move)) {
-         movelist[depth].score[n] += 800;
-         if (move == killer[depth+2][0])
-            movelist[depth].score[n] += 50;
-      } else if (is_counter(depth, prev_move, move)) {
-         movelist[depth].score[n] += 1500;
-      } else if (is_castle_move(move)) {
-         movelist[depth].score[n] += 600;
-      } else if (is_capture_move(move)) {
-         int s = see(move);
-         //max_see = std::max(max_see, s);
-         if (s > 0)
-            movelist[depth].score[n] += 2400 + s / 10;
-         else
-            movelist[depth].score[n] += s;
-
-         if (threat_move && get_move_to(move) == get_move_from(threat_move) && s < 0)
-            movelist[depth].score[n] += 1000;
-      } else if (is_pickup_move(move)) {
-         if (threat_move && get_move_to(threat_move) == get_move_from(move)) {
-            movelist[depth].score[n] += 1500;
-         } else {
-            movelist[depth].score[n] += 0;
-         }
-      } else if (is_drop_move(move)) {
-         int history_score = get_move_history_score(move);
-         int history_scale = get_move_history_scale(move);
-         int s = see(move);
-         if (s > 0) {
-            int h = history_scale ? 100 * history_score / history_scale : 100;
-            if (!(board.bbc[me] & board.royal & bitboard_t<kind>::neighbour_board[get_move_to(move)]).is_empty())
-               movelist[depth].score[n] += 1200 + h;
-            else if (bitboard_t<kind>::board_homeland[next_side[me]].test(get_move_to(move)))
-               movelist[depth].score[n] += 1100 + h;
-            movelist[depth].score[n] += s / 10;
-         } else {
-            int h = history_scale ? 500 * history_score / history_scale : 500;
-            movelist[depth].score[n] += s + h;
-         }
-      } else if (max_history && !board.check() && !is_promotion_move(move) && !is_capture_move(move)) {
-         int history_score = get_move_history_score(move);
-         int history_scale = get_move_history_scale(move);
-         if (history_scale) {
-            int s = 500 * history_score / history_scale;
-            if (s) {
-               movelist[depth].score[n] = 0 + s;
-            } else {
-               s = see(move);
-               max_see = std::max(max_see, s);
-               movelist[depth].score[n] += s;
-            }
-         } else {
-            //int s = see(move);
-            //max_see = std::max(max_see, s);
-            //movelist[depth].score[n] += s + centre_table[get_move_to(move)] - centre_table[get_move_from(move)];
-         }
-
-         if (threat_move && get_move_from(move) == get_move_to(threat_move))
-            movelist[depth].score[n] += 500;
-      } else {
-         int s = see(move);
-         max_see = std::max(max_see, s);
-         movelist[depth].score[n] += s;
-
-         if (threat_move && get_move_from(move) == get_move_to(threat_move))
-            movelist[depth].score[n] += 500;
-      }
-
-      if (is_gate_move(move))
-         movelist[depth].score[n] += 200;
-
-      if (depth < 3 && clock.root_moves_played < 10 && random_ok) {
-         movelist[depth].score[n] += genrandui() & 0xff;
-      }
-   }
-
-   /* FIXME: don't do futility pruning if there are pawns about to promote */
-   //if (depth > 0 && draft < FUTILITY_DEPTH && !board.check() && prev_move != 0 && !is_mate_score(beta)) {
-   //   if (static_score + max_see + draft*100 < alpha)
-   //      return static_score + max_see + draft*100;
-   //}
+   score_moves(depth, hash_move, prev_move, threat_move);
 
    /* Search the first move */
    hash_flag = HASH_TYPE_UPPER;
@@ -685,7 +785,7 @@ int search(int alpha, int beta, int draft, int depth)
       board.check(movegen.was_checking_move(&board, board.side_to_move, move));
       if ((board.rule_flags & RF_USE_SHAKMATE) && board.check()) test_shak();
       int e = get_extension(move, move_score);
-      score = -search(-beta, -alpha, draft-1 + board.check(), depth+1);
+      score = -search(-beta, -alpha, draft-1 + e, depth+1);
 
       if (score < LEGALLOSS) {   /* Illegal move, but requires search to identify */
          legal_moves--;
@@ -713,8 +813,11 @@ int search(int alpha, int beta, int draft, int depth)
       int move_score = movelist[depth].get_move_score();
 
       /* Prune unsafe drops near the tips */
-      if (draft < 3 && move_score < 500 && !in_check && is_drop_move(move)) continue;
-      if (draft < 3 && is_pickup_move(move)) continue;
+      if (is_drop_move(move)) {
+         if (draft < 3 && move_score < 500 && !in_check) continue;
+         //if (draft < 2 && !in_check && !is_mate_score(beta)) continue;
+         if (depth == 0 && draft == 1 && moves_searched) continue;
+      }
       if (draft < 3 && is_pickup_move(move)) continue;
 
       playmove(move);
@@ -727,11 +830,16 @@ int search(int alpha, int beta, int draft, int depth)
       if ((board.rule_flags & RF_USE_SHAKMATE) && board.check()) test_shak();
       clock.nodes_searched++;
 
-      bool avoid_reduction = in_check || board.check() || is_capture_move(move) || is_promotion_move(move) || is_castle_move(move) || is_pawn_push(move);
+      /* Futility pruning of drop moves in frontier nodes */
+      if (draft == 1 && is_drop_move(move) && !in_check && !board.check()) {
+         takeback();
+         continue;
+      }
+
+      bool avoid_reduction = in_check || board.check() || is_capture_move(move) || is_promotion_move(move) || is_castle_move(move) || is_pawn_push(move, me);
 
       int e = get_extension(move, move_score);
-      int r = get_reduction(move, move_score, moves_searched, draft);
-      if (e || avoid_reduction) r = 0;
+      int r = (e || avoid_reduction) ? 0 : get_reduction(move, move_score, moves_searched, draft);
 
       if (depth < 2) {
          score = -search(-beta, -alpha, draft-1 + e-r, depth+1);
@@ -773,7 +881,7 @@ int search(int alpha, int beta, int draft, int depth)
        */
       store_killer(hash_move, depth);
       store_mate_killer(hash_move, depth, best_score);
-      store_counter_move(depth, prev_move, move);
+      store_counter_move(depth, prev_move, hash_move);
       update_history(move, draft * draft);
       if (prev_move == 0)
          store_null_killer(depth, hash_move);
@@ -798,7 +906,6 @@ no_moves:
       hash_move = 0;
       best_score = board.check() ? (mate_score + depth)
                                  : (stale_score + depth);
-      if (best_score == depth) hash_flag = HASH_TYPE_UPPER;
 
       /* Make sure mate wasn't caused by an illegal drop */
       if (board.check() && is_drop_move(prev_move)) {
@@ -807,6 +914,11 @@ no_moves:
             return ILLEGAL;
       }
 
+      if (board.bbc[board.side_to_move].is_empty())
+         best_score = no_piece_score + depth;
+
+      if (best_score == depth) hash_flag = HASH_TYPE_UPPER;
+
       /* Test if the mate is legal, that is to say:
        *  - The checking piece is allowed to deliver mate
        *  - The mate follows a sequence of checks at least one of which
@@ -837,42 +949,40 @@ no_moves:
    return best_score;
 }
 
-void print_principle_variation()
+void print_principle_variation(int d = 0)
 {
    int c;
-   int root_move_number = 0;
+   int root_move_number = start_move_count;
 
    if (board.side_to_move) {
-      root_move_number = 1;
-      iter(" %d. ...", (int)(moves_played)/2+1);
+      root_move_number += 1;
+      iter(" %d. ...", (int)(root_move_number+moves_played)/2+1);
    }
-   for (c=0; c<length_of_variation[0]; c++) {
+   for (c=0; c<length_of_variation[d]; c++) {
       if (((root_move_number+c) & 1) == 0)
          iter(" %d.", (int)(root_move_number+moves_played)/2+1);
       movelist_t movelist;
       generate_legal_moves(&movelist);
-      iter(" %-6s", move_to_short_string(principle_variation[c][0], &movelist));
-      playmove(principle_variation[c][0]);
+      iter(" %-6s", move_to_short_string(principle_variation[c][d], &movelist));
+      playmove(principle_variation[c][d]);
    }
-   for (c=0; c<length_of_variation[0]; c++)
+   for (c=0; c<length_of_variation[d]; c++)
       takeback();
 }
 
-void print_principle_variation_xb()
+void print_principle_variation_xb(movelist_t *movelist = NULL)
 {
    int c;
-   int root_move_number = 0;
+   int root_move_number = start_move_count;
 
    if (board.side_to_move == BLACK) {
-      root_move_number = 1;
-      xb(" %d. ...", (int)(moves_played)/2+1);
+      root_move_number += 1;
+      xb(" %d. ...", (int)(root_move_number+moves_played)/2+1);
    }
    for (c=0; c<length_of_variation[0]; c++) {
       if (((root_move_number+c) & 1) == 0)
          xb(" %d.", (int)(root_move_number+moves_played)/2+1);
-      movelist_t movelist;
-      generate_legal_moves(&movelist);
-      xb(" %s", move_to_short_string(principle_variation[c][0], &movelist));
+      xb(" %s", move_to_short_string(principle_variation[c][0], movelist));
       playmove(principle_variation[c][0]);
    }
    for (c=0; c<length_of_variation[0]; c++)
@@ -933,6 +1043,9 @@ play_state_t get_game_end_state(movelist_t *movelist = NULL)
 
    /* No legal moves? */
    if (movelist->num_moves == 0) {
+      if (board.bbc[board.side_to_move].is_empty())
+         return SEARCH_GAME_ENDED_NOPIECES;
+
       if (!board.check())
          return SEARCH_GAME_ENDED_STALEMATE;
 
@@ -1007,6 +1120,8 @@ play_state_t think(int max_depth)
 
    /* Test if the game has ended */
    state = get_game_end_state(&movelist);
+   if (!repetition_claim && state == SEARCH_GAME_ENDED_REPEAT)
+      state = SEARCH_OK;
    if (state != SEARCH_OK)
       return state;
 
@@ -1024,8 +1139,25 @@ play_state_t think(int max_depth)
 
    /* Iterative deepening loop */
    int e = board.check();
+   clock.extra_time = clock.time_left;
+   best_move[0] = 0;
    int score = search(-LEGALWIN, LEGALWIN, 1, 0);
+   clock.extra_time = 0;
+   if (abort_search) {
+      xb("# Aborted ply 1 search - no move!\n");
+      if (best_move[0] == 0) {
+         xb("# Pick random move\n");
+         best_move[0] = movelist.move[0];
+      }
+
+      length_of_variation[0] = 1;
+      principle_variation[0][0] = best_move[0];
+      xb("% 3d % 5d %6d %9d ", 1, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
+      print_principle_variation_xb(&movelist);
+      xb("\n");
+   }
    move_t move = best_move[0];
+   if (!abort_search)
    for (depth=2; depth<=max_depth; depth++) {
       scale_history();
       movelist.rewind();
@@ -1049,16 +1181,16 @@ play_state_t think(int max_depth)
 
          /* Report a fail-low */
          if (score < alpha && move) {
-            move_t move = this->best_move[0];
+            move_t move = best_move[0];
             if (show_fail_low) {
-               iter("% 3d.  %6.2f   %9d  %+2.2f  % 2d/% 2d ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0, 0, 0);
-               iter(" %d.", (int)(moves_played)/2+1);
+               iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
+               iter(" %d.", (int)(start_move_count + moves_played)/2+1);
                if (board.side_to_move) iter(" ...");
                iter(" %s?", move_to_short_string(move, &movelist));
                iter("\n");
 
                xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-               xb(" %d.", (int)(moves_played)/2+1);
+               xb(" %d.", (int)(start_move_count + moves_played)/2+1);
                if (board.side_to_move) xb(" ...");
                xb(" %s?", move_to_short_string(move, &movelist));
                xb("\n");
@@ -1071,14 +1203,14 @@ play_state_t think(int max_depth)
          if (score >= beta) {
             move = best_move[0];
             if (show_fail_high) {
-               iter("% 3d.  %6.2f   %9d  %+2.2f  % 2d/% 2d ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0, 0, 0);
-               iter(" %d.", (int)(moves_played)/2+1);
+               iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
+               iter(" %d.", (int)(start_move_count + moves_played)/2+1);
                if (board.side_to_move) iter(" ...");
                iter(" %s!", move_to_short_string(move, &movelist));
                iter("\n");
 
                xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-               xb(" %d.", (int)(moves_played)/2+1);
+               xb(" %d.", (int)(start_move_count + moves_played)/2+1);
                if (board.side_to_move) xb(" ...");
                xb(" %s!", move_to_short_string(move, &movelist));
                xb("\n");
@@ -1086,14 +1218,12 @@ play_state_t think(int max_depth)
             clock.extra_time = (5*clock.time_left)/10;
          }
       }
-      iter("% 3d.  %6.2f   %9d  %+2.2f  % 2d/% 2d ", depth, (get_timer() - start_time)/1000000.0, (int)clock.nodes_searched, score/100.0, 0, 0);
-      //iter("%s", move_to_string(move));
+      iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer() - start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
       print_principle_variation();
       iter("\n");
 
       xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-      //xb("%s", move_to_string(move));
-      print_principle_variation_xb();
+      print_principle_variation_xb(&movelist);
       xb("\n");
 
       uci("info depth %d score cp %d nodes %llu time %llu hashfull %d pv",
@@ -1103,9 +1233,9 @@ play_state_t think(int max_depth)
 
       store_principle_variation(score, depth);
 
-      if (is_mate_score(score) && (LEGALWIN - abs(score)) < depth+1 && move) break;
+      if (is_mate_score(score) && (LEGALWIN - abs(score)) == length_of_variation[0] && move && !analysing) break;
       if (abort_search) break;
-      if (movelist.num_moves == 1 && move && !pondering) break;
+      if (movelist.num_moves == 1 && move && !pondering && !analysing) break;
 
       /* Check if we have enough time for the next iteration, assuming an
        * effective branching ratio of 2.
@@ -1114,6 +1244,24 @@ play_state_t think(int max_depth)
          break;
    }
 
+   while (analysing && !abort_search)
+      check_clock();
+
+   if (score < resign_threshold)
+      resign_count++;
+   else
+      resign_count = 0;
+
+   if (abs(score) < draw_threshold)
+      draw_count++;
+   else
+      draw_count = 0;
+
+   if (moves_played > 400)
+      draw_count += 50;
+
+   //printf("# %d [draw %d / %d] [resign %d / %d]\n", score, draw_threshold, draw_count, resign_threshold, resign_count);
+
    assert(move);
    playmove(move);
 
@@ -1174,8 +1322,10 @@ bool analyse()
    while (analysing && !abort_search) {
       if (think(MAX_SEARCH_DEPTH) == SEARCH_OK)
          takeback();
-      else
-         break;
+      else {
+         while (analysing && !abort_search)
+            check_clock();
+      }
    }
    if (analyse_move) playmove(analyse_move);
    analyse_move = 0;
