@@ -2,8 +2,10 @@ int move_mvvlva(move_t move) const
 {
    int value = 0;
 
-   value += pt.see_piece_value[get_move_captured_piece(move)] * 8;
-   value -= pt.see_piece_value[get_move_piece(move)] / 8;
+   if (is_capture_move(move)) {
+      value += pt.see_piece_value[board.get_piece(get_move_capture_square(move))] * 8;
+      value -= pt.see_piece_value[get_move_piece(move)] / 8;
+   }
 
    return value;
 }
@@ -13,7 +15,9 @@ int move_value(move_t move, side_t /* side_to_move */) const
    /* Capture a piece */
    int value = 0;
    if (is_capture_move(move))
-      value += pt.see_piece_value[get_move_captured_piece(move)];
+      value += pt.see_piece_value[board.get_piece(get_move_capture_square(move))];
+   if (is_drop_move(move))
+      value += pt.see_piece_value[get_move_piece(move)];
    if (is_promotion_move(move)) {
       //value += pt.see_piece_value[get_move_promotion_piece(move)];
       //value -= pt.see_piece_value[get_move_piece(move)];
@@ -80,12 +84,12 @@ int see(move_t move)
 {
    int score[128];
    int depth;
-   int side       = get_move_player(move);
+   side_t side    = board.side_to_move;
    int piece      = get_move_piece(move);
    int square     = get_move_to(move);
    int from       = get_move_from(move);
    int last_piece = piece;
-   bitboard_t<kind> attackers, hidden_attackers;
+   bitboard_t<kind> attackers, hidden_attackers, pinned;
    bitboard_t<kind> xray_update;
    bitboard_t<kind> own;
    bitboard_t<kind> mask = bitboard_t<kind>::board_all;
@@ -96,7 +100,6 @@ int see(move_t move)
 
    /* Initial gain: the piece on the target square */
    depth = 0;
-   score[depth] = 0;
    score[depth] = move_value(move, board.side_to_move);
    if (is_capture_move(move)) {
       /* Easy: if the piece we use to capture the enemy piece is worth less
@@ -130,15 +133,19 @@ int see(move_t move)
    if (!is_drop_move(move))
       mask.reset(from);
    attackers = movegen.get_all_attackers(&board, mask, square);
-   hidden_attackers = movegen.super[square];
+   hidden_attackers = xray_update;
    hidden_attackers &= ~attackers;
-   hidden_attackers &= xray_update;
+
+   //pinned = movegen.get_pinned_pieces(&board, next_side[side]) | movegen.get_pinned_pieces(&board, side);
+   //attackers &= ~pinned;
+   //hidden_attackers &= ~pinned;
+   //mask &= ~pinned;
 
    do {
       assert(depth < 128);
       /* Consider next move - one ply deeper, flip side */
       depth++;
-      side ^= BLACK;
+      side = next_side[side];
       /* Update score (speculative) */
       score[depth] = pt.piece_value[last_piece] - score[depth-1];
       /* Perform LVA capture */
